# -*- coding: utf-8 -*-

from imio.smartweb.locales import SmartwebMessageFactory as _
from plone import api
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class ContactTypeVocabularyFactory:
    def __call__(self, context=None):
        values = [
            (
                "organization",
                _(
                    "Organization (administrative service, business, professional, sports club, association, etc.)"
                ),
            ),
            (
                "position",
                _(
                    "Position (mayor, alderman, advisor, director, head of department, etc.)"
                ),
            ),
            ("mission", _("Mission (passports, reception, parking, etc.)")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


ContactTypeVocabulary = ContactTypeVocabularyFactory()


class PhoneTypeVocabularyFactory:
    def __call__(self, context=None):
        """vcard spec : cell, home, work, text, voice, fax, video, pager, textphone"""
        values = [
            ("fax", _("Fax")),
            ("cell", _("Mobile")),
            ("home", _("Personal phone")),
            ("work", _("Work phone")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


PhoneTypeVocabulary = PhoneTypeVocabularyFactory()


class MailTypeVocabularyFactory:
    def __call__(self, context=None):
        """vcard spec : home, work"""
        values = [
            ("home", _("Personal email")),
            ("work", _("Work email")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


MailTypeVocabulary = MailTypeVocabularyFactory()


class SiteTypeVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("facebook", _("Facebook")),
            ("instagram", _("Instagram")),
            ("pinterest", _("Pinterest")),
            ("twitter", _("Twitter")),
            ("website", _("Website")),
            ("youtube", _("Youtube")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


SiteTypeVocabulary = SiteTypeVocabularyFactory()


class FacilitiesVocabularyFactory:
    def __call__(self, context=None):
        values = [
            ("accessibility", _("Accessibility (PMR)")),
            ("defibrillator", _("Defibrillator")),
            ("drinking_water_point", _("Drinking water point")),
            ("useful_numbers", _("Useful numbers")),
            ("public_toilets", _("Public toilets")),
            ("free_wifi", _("Free WIFI")),
        ]
        terms = [SimpleTerm(value=t[0], token=t[0], title=t[1]) for t in values]
        return SimpleVocabulary(terms)


FacilitiesVocabulary = FacilitiesVocabularyFactory()


class EntitiesUIDsVocabularyFactory:
    def __call__(self, context=None):
        portal = api.portal.get()
        brains = api.content.find(
            context=portal,
            portal_type="imio.directory.Entity",
            sort_on="sortable_title",
        )
        terms = [SimpleTerm(value=b.UID, token=b.UID, title=b.Title) for b in brains]
        return SimpleVocabulary(terms)


EntitiesUIDsVocabulary = EntitiesUIDsVocabularyFactory()



# -*- coding: utf-8 -*-

from collective.geolocationbehavior.geolocation import IGeolocatable
from imio.directory.core.config import PHONE_RE
from imio.directory.core.config import URL_RE
from imio.smartweb.common.utils import geocode_object
from imio.smartweb.common.utils import get_vocabulary
from imio.smartweb.locales import SmartwebMessageFactory as _
from io import StringIO
from plone import api
from plone.autoform.form import AutoExtensibleForm
from plone.formwidget.geolocation.geolocation import Geolocation
from plone.namedfile.field import NamedFile
from Products.CMFPlone.PloneTool import EMAIL_RE
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import button
from z3c.form.form import Form
from z3c.form.interfaces import NOT_CHANGED
from zope.globalrequest import getRequest
from zope.i18n import translate
from zope.interface import Interface

import csv
import logging
import re

logger = logging.getLogger("imio.directory.core")


def convert_empty_val(value):
    """Ensure empty values are all None"""
    if value in ["---", ""]:
        # --- is used as blank field in sheet template listbox
        return None
    return str(value).strip()


def cleanup(lst):
    """Remove duplicates and empty values from list"""
    lst = list(set(lst))
    lst = [val for val in lst if convert_empty_val(val)]
    return lst


def is_in_vocab(voc_name, value):
    """Check if value is in specified vocabulary"""
    voc = get_vocabulary(voc_name)
    terms_token = [term.token for term in voc]
    return value in terms_token


def validate_type(value):
    if not value:
        return
    elif not is_in_vocab("imio.directory.vocabulary.ContactTypes", value):
        return _("Chosen contact type '${val}' doesn't exist.", mapping={"val": value})


def validate_country(value):
    if not value:
        return
    elif not is_in_vocab("imio.smartweb.vocabulary.Countries", value):
        return _(
            "Chosen country '${val}' doesn't exist (good format sample: be, fr).",
            mapping={"val": value},
        )


def validate_phone(value, value_type):
    errors = []
    if re.match(PHONE_RE, value) is None:
        msg = _(
            "Bad phone format '${val}' (good format sample: +32444556677).",
            mapping={"val": value},
        )
        errors.append(msg)

    if not is_in_vocab("imio.directory.vocabulary.PhoneTypes", value_type):
        msg = _(
            "Chosen phone type '${val}' doesn't exist.",
            mapping={"val": value_type},
        )
        errors.append(msg)
    return errors


def validate_mail(value, value_type):
    errors = []
    if re.match(EMAIL_RE, value) is None:
        msg = _(
            "Bad mail format '${val}' (good format sample: noreply@imio.be).",
            mapping={"val": value},
        )
        errors.append(msg)

    if not is_in_vocab("imio.directory.vocabulary.MailTypes", value_type):
        msg = _(
            "Chosen mail type '${val}' doesn't exist.",
            mapping={"val": value_type},
        )
        errors.append(msg)
    return errors


def validate_url(value, value_type):
    errors = []
    if re.match(URL_RE, value) is None:
        msg = _(
            "Bad URL format '${val}' (good format sample: https://www.imio.be).",
            mapping={"val": value},
        )
        errors.append(msg)

    if not is_in_vocab("imio.directory.vocabulary.SiteTypes", value_type):
        msg = _(
            "Chosen URL type '${val}' doesn't exist.",
            mapping={"val": value_type},
        )
        errors.append(msg)
    return errors


class ContactRow(object):
    def __init__(self, contact_row):
        self.contact_type = convert_empty_val(contact_row[0])
        self.title = convert_empty_val(contact_row[1])
        self.subtitle = convert_empty_val(contact_row[2])
        self.description = convert_empty_val(contact_row[3])
        self.street = convert_empty_val(contact_row[4])
        self.number = convert_empty_val(contact_row[5])
        self.complement = convert_empty_val(contact_row[6])
        self.zipcode = convert_empty_val(contact_row[7])
        self.city = convert_empty_val(contact_row[8])
        self.country = convert_empty_val(contact_row[9])
        self.vat_number = convert_empty_val(contact_row[10])
        self.latitude = convert_empty_val(contact_row[11])
        self.longitude = convert_empty_val(contact_row[12])
        self.phones = [
            {
                "label": contact_row[13],
                "type": contact_row[14],
                "number": contact_row[15],
            },
            {
                "label": contact_row[16],
                "type": contact_row[17],
                "number": contact_row[18],
            },
            {
                "label": contact_row[19],
                "type": contact_row[20],
                "number": contact_row[21],
            },
        ]
        self.phones = [p for p in self.phones if convert_empty_val(p.get("number"))]
        for phone in self.phones:
            phone["number"] = phone["number"].replace(" ", "")
            if not phone["number"].startswith("+"):
                # add '+' in phone number if missing
                phone["number"] = f"+{phone['number']}"
        self.mails = [
            {
                "label": contact_row[22],
                "type": contact_row[23],
                "mail_address": contact_row[24],
            },
            {
                "label": contact_row[25],
                "type": contact_row[26],
                "mail_address": contact_row[27],
            },
            {
                "label": contact_row[28],
                "type": contact_row[29],
                "mail_address": contact_row[30],
            },
        ]
        self.mails = [m for m in self.mails if convert_empty_val(m.get("mail_address"))]
        for mail in self.mails:
            mail["mail_address"] = mail["mail_address"].strip()
        self.urls = [
            {"type": contact_row[31], "url": contact_row[32]},
            {"type": contact_row[33], "url": contact_row[34]},
            {"type": contact_row[35], "url": contact_row[36]},
        ]
        self.urls = [u for u in self.urls if convert_empty_val(u.get("url"))]
        for url in self.urls:
            url["url"] = url["url"].strip()
            if not url["url"].startswith("http"):
                # add 'https://' in website url if missing
                url["url"] = f"https://{url['url']}"
        self.topics = cleanup([contact_row[37], contact_row[38], contact_row[39]])
        # XXX categories are not inserted yet
        self.categories = cleanup([contact_row[40], contact_row[41], contact_row[42]])
        self.facilities = cleanup([contact_row[43], contact_row[44], contact_row[45]])
        self.iam = cleanup([contact_row[46], contact_row[47], contact_row[48]])

    def validate(self):
        errors = []
        if self.contact_type is None:
            errors.append(_("Contact type is missing."))
        else:
            errors.append(validate_type(self.contact_type))

        if self.title is None:
            errors.append(_("Contact title is missing."))

        if self.zipcode is not None and not self.zipcode.isdigit():
            errors.append(
                _("Zipcode '${val}' is not a number.", mapping={"val": self.zipcode})
            )

        errors.append(validate_country(self.country))

        for phone in self.phones:
            errors += validate_phone(phone.get("number"), phone.get("type"))

        for mail in self.mails:
            errors += validate_mail(mail.get("mail_address"), mail.get("type"))

        for url in self.urls:
            errors += validate_url(url.get("url"), url.get("type"))

        for topic in self.topics:
            if not is_in_vocab("imio.smartweb.vocabulary.Topics", topic):
                errors.append(
                    _("Chosen topic '${val}' doesn't exist.", mapping={"val": topic})
                )

        for facility in self.facilities:
            if not is_in_vocab("imio.directory.vocabulary.Facilities", facility):
                errors.append(
                    _(
                        "Chosen facility '${val}' doesn't exist.",
                        mapping={"val": facility},
                    )
                )

        for iam in self.iam:
            if not is_in_vocab("imio.smartweb.vocabulary.IAm", iam):
                errors.append(
                    _("Chosen i am term '${val}' doesn't exist.", mapping={"val": iam})
                )

        errors = [e for e in errors if e]
        return errors


class ContactsImporter(object):
    contact_rows = []

    def __init__(self, data):
        self.contact_rows = []
        reader = csv.reader(
            StringIO(data.decode("utf-8")),
            delimiter=";",
            dialect="excel",
            quotechar='"',
        )
        # Skip csv header row
        next(reader)
        for row in reader:
            contact_row = ContactRow(row)
            self.contact_rows.append(contact_row)

    def validate(self):
        errors = {}
        for index, contact_row in enumerate(self.contact_rows, start=1):
            row_errors = contact_row.validate()
            if row_errors:
                errors[index] = [translate(r, context=getRequest()) for r in row_errors]
        return errors

    def create_contact(self, contact_row, container):
        contact = api.content.create(
            container=container,
            type="imio.directory.Contact",
            title=contact_row.title,
        )
        contact.type = contact_row.contact_type
        contact.subtitle = contact_row.subtitle
        contact.description = contact_row.description
        contact.street = contact_row.street
        contact.number = contact_row.number
        contact.complement = contact_row.complement
        if contact_row.zipcode:
            contact.zipcode = int(contact_row.zipcode.strip())
        contact.city = contact_row.city
        contact.country = contact_row.country
        contact.vat_number = contact_row.vat_number
        if contact_row.latitude and contact_row.longitude:
            IGeolocatable(contact).geolocation = Geolocation(
                latitude=contact_row.latitude, longitude=contact_row.longitude
            )
        contact.phones = contact_row.phones
        contact.mails = contact_row.mails
        contact.urls = contact_row.urls
        contact.topics = contact_row.topics
        contact.facilities = contact_row.facilities
        contact.iam = contact_row.iam
        geocode_object(contact)
        contact.reindexObject()

    def create_contacts(self, container):
        for contact_row in self.contact_rows:
            self.create_contact(contact_row, container)
        return len(self.contact_rows)


class IImportCSVFormSchema(Interface):
    csv_file = NamedFile(title=_("CSV file"))


class ImportContactForm(AutoExtensibleForm, Form):
    label = _("Import contacts")
    schema = IImportCSVFormSchema
    template = ViewPageTemplateFile("import.pt")
    ignoreContext = True

    @button.buttonAndHandler(_("Import"), name="import")
    def action_import(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return

        csv_file = data["csv_file"]
        if csv_file is NOT_CHANGED:
            # avoid traceback or double treatment
            return

        importer = ContactsImporter(csv_file.data)

        # validate contacts values
        errors = importer.validate()
        if errors:
            self.status = self.formErrorsMessage
            self.errors_details = errors
        else:
            # import contacts
            number = importer.create_contacts(self.context)
            self.status = _(
                "Imported ${number} contacts successfully !", mapping={"number": number}
            )



# -*- coding: utf-8 -*-

from plone import api
from plone.app.contenttypes.browser.folder import FolderView
from plone.dexterity.browser.view import DefaultView
from Products.CMFPlone.resources import add_bundle_on_request
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


class ContactView(DefaultView, FolderView):
    """imio.directory.core.Contact view
    FolderView brings get_thumb_scale_list
    """

    def __call__(self):
        images = self.context.listFolderContents(contentFilter={"portal_type": "Image"})
        if len(images) > 0:
            add_bundle_on_request(self.request, "spotlightjs")
            add_bundle_on_request(self.request, "flexbin")
        self.update()
        return self.index()

    def files(self):
        return self.context.listFolderContents(contentFilter={"portal_type": "File"})

    def images(self):
        return self.context.listFolderContents(contentFilter={"portal_type": "Image"})

    def sub_contacts(self):
        factory = getUtility(
            IVocabularyFactory, "imio.directory.vocabulary.ContactTypes"
        )
        vocabulary = factory()
        sub_contacts = {}
        for term in vocabulary:
            sub_contacts_for_type = api.content.find(
                context=self.context,
                depth=1,
                portal_type="imio.directory.Contact",
                contact_type=term.value,
                sort_on="sortable_title",
            )
            if sub_contacts_for_type:
                sub_contacts[term.title] = sub_contacts_for_type
        return sub_contacts



# -*- coding: utf-8 -*-

from imio.directory.core.utils import get_entity_uid_for_contact
from imio.smartweb.common.faceted.utils import configure_faceted
from imio.smartweb.common.interfaces import IAddress
from imio.smartweb.common.utils import geocode_object
from zope.component import getMultiAdapter
from zope.globalrequest import getRequest
from zope.lifecycleevent.interfaces import IAttributes

import os


def set_default_entity_uid(contact):
    contact.selected_entities = contact.selected_entities or []
    uid = get_entity_uid_for_contact(contact)
    if uid not in contact.selected_entities:
        contact.selected_entities = contact.selected_entities + [uid]
        contact.reindexObject()


def added_entity(obj, event):
    faceted_config_path = "{}/faceted/config/entity.xml".format(
        os.path.dirname(__file__)
    )
    configure_faceted(obj, faceted_config_path)
    request = getRequest()
    request.form = {
        "cid": "entity",
        "faceted.entity.default": obj.UID(),
    }
    handler = getMultiAdapter((obj, request), name="faceted_update_criterion")
    handler.edit(**request.form)


def added_contact(obj, event):
    set_default_entity_uid(obj)
    if not obj.is_geolocated:
        # geocode only if the user has not already changed geolocation
        geocode_object(obj)


def modified_contact(obj, event):
    set_default_entity_uid(obj)

    if not hasattr(event, "descriptions") or not event.descriptions:
        return
    for d in event.descriptions:
        if not IAttributes.providedBy(d):
            # we do not have fields change description, but maybe a request
            continue
        if d.interface is IAddress and d.attributes:
            # an address field has been changed
            geocode_object(obj)
            return



# -*- coding: utf-8 -*-

from imio.directory.core.testing import IMIO_DIRECTORY_CORE_INTEGRATION_TESTING
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.namedfile.file import NamedBlobImage
from zope.component import getMultiAdapter

import os
import unittest


def image(filename):
    file_path = os.path.join(os.path.dirname(__file__), filename)
    return NamedBlobImage(data=open(file_path, "rb").read(), filename=file_path)


class TestUtils(unittest.TestCase):

    layer = IMIO_DIRECTORY_CORE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests"""
        self.request = self.layer["request"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.entity = api.content.create(
            container=self.portal,
            type="imio.directory.Entity",
            title="Entity",
        )

    def test_export_to_vcard(self):
        contact = api.content.create(
            container=self.entity,
            type="imio.directory.Contact",
            title="contact",
        )
        contact.street = "My street"
        contact.number = "1"
        contact.zipcode = 5000
        contact.country = "be"
        view = getMultiAdapter((contact, self.request), name="utils")
        self.assertTrue(view.can_export_contact_to_vcard())
        self.assertEqual(
            str(view.export_contact_to_vcard()),
            "BEGIN:VCARD\r\nVERSION:3.0\r\nADR:1;;My street;;;5000;Belgium\r\nFN:contact\r\nEND:VCARD\r\n",
        )
        contact.phones = [{"label": "label", "type": "cell", "number": "+32496111111"}]
        contact.mails = [
            {"label": "label", "type": "home", "mail_address": "test@imio.be"}
        ]
        self.assertEqual(
            str(view.export_contact_to_vcard()),
            "BEGIN:VCARD\r\nVERSION:3.0\r\nADR:1;;My street;;;5000;Belgium\r\nEMAIL;TYPE=home:test@imio.be\r\nFN:contact\r\nTEL;TYPE=cell:+32496111111\r\nEND:VCARD\r\n",
        )
        contact.urls = [{"type": "website", "url": "https://www.imio.be"}]
        self.assertEqual(
            str(view.export_contact_to_vcard()),
            "BEGIN:VCARD\r\nVERSION:3.0\r\nADR:1;;My street;;;5000;Belgium\r\nEMAIL;TYPE=home:test@imio.be\r\nFN:contact\r\nTEL;TYPE=cell:+32496111111\r\nURL;TYPE=website:https://www.imio.be\r\nEND:VCARD\r\n",
        )
        contact.logo = image("resources/logo.png")
        self.assertIn(
            "PHOTO;ENCODING=B;TYPE=IMAGE/JPEG:", view.export_contact_to_vcard()
        )

        view = getMultiAdapter((self.portal, self.request), name="utils")
        self.assertFalse(view.can_export_contact_to_vcard())
        self.assertIsNone(view.export_contact_to_vcard())

        view = getMultiAdapter((self.entity, self.request), name="utils")
        self.assertFalse(view.can_export_contact_to_vcard())
        self.assertIsNone(view.export_contact_to_vcard())
